let initBlockHeight = 200;
$(document).ready(function()
{
    setTimeout(function()
    {
        const headerH = $('#header').height() + $('.detail-header').height() + $('.wizard-introduction').height() + 66;
        $('.result-table').css('max-height', 'calc(100vh - ' + headerH + 'px)');

        if($('#runResult').length)
        {
            let resultCanvas = $('#runResult').clone();
            resultCanvas.attr('id', 'resultCanvas').addClass('hidden');
            resultCanvas.find('.think-model-content, .model-block').addClass('is-export');
            if(resultCanvas.find('#thinkrun-result-table').length) resultCanvas.find('#thinkrun-result-table').remove();
            $('#runResult').after(resultCanvas);
        }

        if($('.think-graph-swot').length)
        {
            const topHeight    = $('.think-result-content .block-0').outerHeight();
            const bottomHeight = $('.think-result-content .block-2').outerHeight();
            const maxHeight    = Math.max(topHeight, bottomHeight);
            $('.think-result-content .block-0, .think-result-content .block-1').css('height', maxHeight);
            $('.think-result-content .block-2, .think-result-content .block-3').css('height', maxHeight);
        }

        if($('.think-graph-pffa').length) calcHeightOfPffa();
    }, 50);

    $('.m-thinkrun-result #runResult .panel-think-graph').on('mouseenter', function()
    {
        if(!localStorage.getItem('showGraphTips'))
        {
            zui.Messager.show({content: canvasZoomTips, close: false});
            localStorage.setItem('showGraphTips', true);
        }
    });

    /* 使用Ctrl+滚轮进行画布缩放。 */
    /* Use Ctrl+scroll wheel to scale the canvas. */
    $('.m-thinkrun-result #runResult').on('wheel', function(event)
    {
        if(type == 'graph' && event.ctrlKey)
        {
            event.preventDefault();

            const deltaY     = event.deltaY;
            const factor     = 0.05;
            let currentScale = $('#runResult .think-graph').css('transform').split(',')[3] || 1;
            currentScale     = parseFloat(currentScale);

            let scale = deltaY < 0 ? (currentScale + factor) : (currentScale - factor);
            scale     = Math.max(0.02, scale);
            $('#runResult .think-graph').css({'transform': `scale(${scale})`, 'transform-origin': 'top center'});
        }
    });

    /* 画布的拖动 */
    /* Dragging the canvas */
    if(type == 'graph' && !$('#runResult .think-graph').zui('moveable')) new zui.Moveable('#runResult .think-graph', {selector: '.think-result-content'});
});

let scriptLoadedMap = {};
/**
 * 获取导出的远程脚本。
 * Get remote script for export.
 *
 * @param  string   url
 * @param  function successCallback
 * @param  function errorCallback
 * @access public
 * @return void
 */
function getRemoteScript(url, successCallback, errorCallback)
{
    if(scriptLoadedMap[url]) return successCallback && successCallback();

    scriptLoadedMap[url] = true;

    let script = document.createElement('script');
    script.type = 'text/javascript';
    script.onload = function() { successCallback(); };
    script.src = url;
    document.head.appendChild(script);
}

/**
 * 更新进度。
 * Update progress.
 *
 * @param  number progress
 * @access public
 * @return void
 */
function updateProgress(progress)
{
    let progressText = tipsLang.exporting;
    if(progress < 1) progressText += Math.floor(progress * 100) + '%';
    $('#mainContent').attr('data-loading', progressText);
}

/**
 * 计算波特五力模型的高度。
 * Calculate the height of pffa think model.
 *
 * @param  boolean isExport
 * @access public
 * @return void
 */
function calcHeightOfPffa(isExport)
{
    let topHeight    = $('.think-result-content .block-0').outerHeight();
    let bottomHeight = $('.think-result-content .block-3').outerHeight();

    if(isExport)
    {
        topHeight    = $('#resultCanvas .think-result-content .block-0 .block-content').outerHeight();
        bottomHeight = $('#resultCanvas .think-result-content .block-3 .block-content').outerHeight();
    }

    let maxHeight = Math.max(topHeight, bottomHeight);
    if(isExport)
    {
        maxHeight = maxHeight + 80;
    }
    else
    {
        initBlockHeight = maxHeight;
    }

    $('#resultCanvas .think-result-content .block-0').css('height', maxHeight + 16);
    $('#resultCanvas .think-result-content .block-3').css('height', maxHeight + 16);
    $('#resultCanvas .think-result-content .block-0>div').css('height', maxHeight);
    $('#resultCanvas .think-result-content .block-3>div').css('height', maxHeight);
}

/**
 * 计算区块宽度。
 * Calculate the block width.
 *
 * @access public
 * @return void
 */
function calcBlockWidth()
{
    const multicolumns = $('#resultCanvas .card-multicolumn');
    const isSwot       = $('.think-graph-swot').length;
    const isPestel     = $('.think-graph-pestel').length;
    const isPest       = $('.think-graph-pest').length;
    const isPffa       = $('.think-graph-pffa').length;
    const is4p         = $('.think-graph-4p').length || $('.think-graph-4p2').length ;
    const widthData    = [];
    multicolumns.each(function()
    {
        let width = $(this).find('.form-batch-table').outerWidth();
        width     = 256 + width;
        widthData.push(width);
        $(this).css('width', width + 'px');
    });
    const maxWidth = Math.max(...widthData) + 30;
    $('#resultCanvas div[class^="block-"], #resultCanvas div[class*="block-"]').css('width', maxWidth + 'px');
    if(isPest || isPestel)
    {
        const modelWidth = isPest ? maxWidth * 4 : maxWidth * 6;
        $('#resultCanvas .think-model-content').css('width', `${modelWidth}px`);
    }
    if(isSwot) $('#resultCanvas .model-swot ').css('width', `${maxWidth * 2}px`);
    if(is4p)   $('#resultCanvas .think-model-content').css('width', `${maxWidth * 2}px`);
    if(isPffa)
    {
        $('#resultCanvas .model-pffa').css('width', `${maxWidth * 3}px`);
        setTimeout(() => calcHeightOfPffa(true), 150);
    }
}

/**
 * 重置画布样式。
 * Reset canvas style.
 *
 * @access public
 * @return void
 */
function resetCanvasStyle()
{
    $('#resultCanvas').removeClass('hidden');
    calcBlockWidth();
    if($('#resultCanvas .think-graph-swot').length)
    {
        const height        = $('#runResult .think-result-content .block-0>div').outerHeight();
        const contentHeight = $('#resultCanvas .think-result-content .block-0>div>div:last-child').outerHeight() + 80;
        const offset        = Math.abs(contentHeight - height);
        $('#resultCanvas .think-result-content .block-0, #resultCanvas .think-result-content .block-1, #resultCanvas .think-result-content .block-2, #resultCanvas .think-result-content .block-3').css('height', height + offset);
    }

    if($('#resultCanvas .think-graph-pffa').length)
    {
        const height = $('#runResult .think-result-content .block-0>div').outerHeight();
        if(height > 500)
        {
            const offset = Math.abs(height - 1000);
            $('#resultCanvas .think-result-content .block-0, #resultCanvas .think-result-content .block-3').css('height', (height + offset * 2));
            $('#resultCanvas .think-result-content .block-0>div, #resultCanvas .think-result-content .block-3>div').css('height', height + offset * 2);
        }
    }

    $('#resultCanvas .think-graph').css({transform: 'scale(0.9)'});
    if($('#resultCanvas .think-graph').outerWidth() > 1200) $('#resultCanvas .think-graph').css({transform: 'scale(0.7)'});
    if($('#resultCanvas .think-graph').outerWidth() > 5000) $('#resultCanvas .think-graph').css({transform: 'scale(0.4)'});
    $('#resultCanvas .think-result-content textarea.form-control').css('border', '1px solid var(--color-gray-100)');
    $('#resultCanvas .think-result-content .form-batch-table .form-control').css('border', '1px solid var(--color-gray-100)');
    $('#resultCanvas *').css('box-shadow', 'none');
    if($('.think-form-batch').length) $('.think-form-batch thead>tr>th>span').css({'height':'28px', 'display': 'block'});

    const offset   = $('.think-graph-pffa').length ? 50 : 38;
    const imgWidth = $('#resultCanvas .think-model-content').width() + offset;
    let imgHeight  = $('#resultCanvas .think-graph').height();
    if($('#resultCanvas .think-graph-pffa').length) imgHeight += 300;
    $('#resultCanvas').css({height: imgHeight, width: imgWidth, background: 'var(--color-gray-50)'});
    $('#resultCanvas .think-graph').css({'height': imgHeight});
    $('html').css('overflow', 'hidden');
    $('#mainContent').css({height: '50%'});
}

/**
 * 绘制页面成画布。
 * Draw the page to a canvas.
 *
 * @param  function successCallback
 * @param  function errorCallback
 * @access public
 * @return void
 */
function drawToCanvas(successCallback, errorCallback)
{
    updateProgress(0);
    resetCanvasStyle();

    let $modelContainer = $('#resultCanvas');
    let canvasHeight    = $modelContainer.outerHeight();
    let canvasWidth     = $modelContainer.outerWidth();
    getRemoteScript(jsRoot + 'js/html2canvas/min.js', function()
    {
        updateProgress(0.2);
        let afterFinish = function(canvas)
        {
            $modelContainer.css({width: '', height: '', background: ''});
            $('#resultCanvas .think-graph').css({height: ''});
            $('#resultCanvas .think-result-content textarea.form-control').css('border', '');
            $('#resultCanvas .think-result-content .form-batch-table .form-control').css('border', '');
            $('html').css('overflow', '');
            $('#mainContent').css({height: ''});
            $('#mainContent #modelDownload').remove();
            if($('.think-form-batch').length) $('.think-form-batch thead>tr>th>span').css({'height': '', 'display': ''});
            $('#resultCanvas .card-multicolumn').css('width', '256px');
            if($('.think-graph-pestel').length) $('div[class^="block-"],div[class*="block-"]').css('width', '544px');
            if($('.think-graph-pffa').length)
            {
                $('#resultCanvas div[class^="block-"], #resultCanvas div[class*="block-"]').css('width', '1078px');
                $('#resultCanvas .think-result-content .block-0').css('height', initBlockHeight + 16);
                $('#resultCanvas .think-result-content .block-3').css('height', initBlockHeight + 16);
                $('#resultCanvas .think-result-content .block-0>div').css('height', initBlockHeight);
                $('#resultCanvas .think-result-content .block-3>div').css('height', initBlockHeight);
            }

            if(!canvas) return;
            try
            {
                canvas.removeNode(true)
            }
            catch(err)
            {
                canvas.remove()
            };
        };
        let delayTime = Math.max(1000, Math.floor(10 * (canvasHeight * canvasWidth) / 100000));
        let progressTimer;
        if(delayTime > 1500)
        {
            let startProgress = 0.2;
            let deltaProgress = 0.5 / Math.floor(delayTime/1000);
            progressTimer = setInterval(function()
            {
                startProgress += deltaProgress;
                updateProgress(Math.min(0.7, startProgress));
            }, 1000);
        }
        setTimeout(function()
        {
            if(progressTimer) clearInterval(progressTimer);
            updateProgress(0.7);
            html2canvas($modelContainer[0], {logging: false, scale: 1}).then(function(canvas)
            {
                updateProgress(0.9);
                canvas.onerror = function()
                {
                    afterFinish(canvas);
                    if(errorCallback) errorCallback('Cannot convert image to blob.');
                };

                canvas.toBlob(function(blob)
                {
                    updateProgress(0.95);
                    let imageUrl = URL.createObjectURL(blob);
                    if(navigator.msSaveBlob)
                    {
                        navigator.msSaveOrOpenBlob(blob, wizardName + '.png');
                    }
                    else
                    {
                        $('#mainContent').append('<a id="modelDownload" style="display:none;" download="' + wizardName + '.png" target="_blank" href="' + imageUrl + '"></a>');
                        $('#mainContent #modelDownload')[0].click();
                    }
                    $('#resultCanvas').addClass('hidden');
                    if(successCallback) successCallback(imageUrl);
                    afterFinish(canvas);
                });
            }).catch(function(error)
            {
                $('#resultCanvas').addClass('hidden');
                afterFinish();
                if(errorCallback) errorCallback('Cannot draw graphic to canvas.');
            });
        }, delayTime);
    }, errorCallback);
}

/**
 * 导出图形画布。
 * Export graph.
 *
 * @access public
 * @return void
 */
function exportGraph()
{
    let $mainContent = $('#mainContent');
    $mainContent.addClass('load-indicator').addClass('loading').css('height', Math.max(200, Math.floor($(window).height() - $('#header').outerHeight() - $('#mainMenu').outerHeight())));
    $('#main').removeClass('load-indicator');
    let afterFinish = function(url)
    {
        setTimeout(function()
        {
            $mainContent.css('height', '').removeClass('loading').removeClass('load-indicator');
            $('#main').addClass('load-indicator');
        }, 300);
        updateProgress(1);
    };
    drawToCanvas(afterFinish, function(errorText)
    {
        afterFinish();
        zui.Messager.show({content: tipsLang.exportFail + (errorText || ''), type: 'danger-outline'});
    });
}
