$(document).ready(function(){
    setTimeout(function() {
        disabledSelectedColumn();
        setLineIndex();
    },50)
})

/**
 * 修改选中项关联区块。
 * Change the selected item is associated with a block..
 *
 * @access public
 * @return void
 */
window.changeSelectedBlock = function ()
{
    const unSelectedItem    = $('.selected-block').data('unSelectedItem');
    const selectedBlock     = $('.selected-block.picker-box').zui('picker').$.value;
    const newUnSelectedItem = [];

    unSelectedItem.forEach(function (item, index) {
        newUnSelectedItem[index] = { ...item };
        newUnSelectedItem[index].disabled = newUnSelectedItem[index].value == selectedBlock;
    });
    $('.unselected-block.picker-box').zui('picker').render({ items: newUnSelectedItem });
}

/**
 * 修改非选中项关联区块。
 * Change non selected item is associated with a block..
 *
 * @access public
 * @return void
 */
window.changeUnSelectedBlock = function ()
{
    const selectedItem    = $('.selected-block').data('unSelectedItem');
    const unSelectedBlock = $('.unselected-block.picker-box').zui('picker').$.value;
    const newSelectedItem = [];

    selectedItem.forEach(function (item, index) {
        newSelectedItem[index] = { ...item };
        newSelectedItem[index].disabled = newSelectedItem[index].value == unSelectedBlock;
    });
    $('.selected-block.picker-box').zui('picker').render({ items: newSelectedItem });
}

/**
 * 点击错误提示的下拉选择框。
 * Click the drop-down box next to the error message.
 *
 * @access public
 * @return void
 */
$(document).on('click', '.link-block-error', function()
{
    $('.link-block-error').removeClass('link-block-error');
    $('.link-tips').addClass('hidden');
});

/**
 * 设置行号。
 * Set line number.
 *
 * @access public
 * @return void
 */
function setLineIndex()
{
    const conditionCount = $('.multicolumn-link-item').length;
    const columnsCount   = $('.multicolumn-link-template').data('columnOption').length;
    let index            = 0;

    $('.multicolumn-link-item').each(function()
    {
        $(this).find('.select-column').picker({name: 'column[' + index + ']'});
        $(this).find('.multicolumn-block').picker({name: 'columnBlock[' + index + ']'});
        index ++;
    })

    /* 多列填空题列关联区块时，添加的条件数大于等于列数的时候，置灰不可添加；当只剩一个条件时不可删除。*/
    /* When multiple column fill in the blank questions are associated with blocks and the number of conditions added is greater than or equal to the number of columns, they are grayed out and cannot be added; Cannot be deleted when there is only one condition left. */
    if(conditionCount >= columnsCount) $('.add-condition').attr('disabled', true);
    if(conditionCount < columnsCount) $('.add-condition').removeAttr('disabled');
    if(conditionCount == 1) $('.remove-condition').attr('disabled', true);
    if(conditionCount > 1)
    {
        $('.remove-condition').removeAttr('disabled');
        $('.remove-condition').removeClass('disabled');
    }
}

/**
 * 添加条件。
 * Add conditions.
 *
 * @access public
 * @return void
 */
window.addCondition = function()
{
    const $newCondition     = $('.multicolumn-link-template').clone(true);
    const index             = $('.multicolumn-link-item').length;
    const multicolumnParams = $('.multicolumn-link-template').data();
    const uniqueColumnId    = 'zin_thinkstep_link_picker_3_column' + new Date().getTime() + '';
    const uniqueBlockId     = 'zin_thinkstep_link_picker_3_block' + new Date().getTime() + '';

    $newCondition.removeClass('hidden').removeClass('multicolumn-link-template').addClass('multicolumn-link-item');
    $newCondition.find('.remove-condition').on('click', function(){removeCondition(this)});
    $('.multicolumn-link-template').before($newCondition);
    setLineIndex();
    /* 实例化选择列和选择区块的picker下拉组件。Instantiate the picker dropdown component for selecting columns and blocks. */
    $newCondition.find('.select-column').each(function() {$(this).attr('id', uniqueColumnId).empty();})
    $newCondition.find('.multicolumn-block').each(function() {$(this).attr('id', uniqueBlockId).empty();})

    $('#' + uniqueColumnId).picker({items: multicolumnParams.columnOption, placeholder: multicolumnParams.selectColumnPlaceholder, name: 'column[' + index + ']', afterRender: function(){disabledSelectedColumn()}, onChange: function(){disabledSelectedColumn()}});
    $('#' + uniqueBlockId).picker({items: multicolumnParams.blocksOptions, placeholder: multicolumnParams.selectBlockPlaceholder, name: 'columnBlock[' + index + ']'});
}

function disabledSelectedColumn()
{
    let selectedVal = [];
    let $pickers    = [];

    $("form .multicolumn-link-item .select-column").each(function()
    {
        let $picker = $(this).zui('picker');
        let value   = $picker.$.value;

        $pickers.push($picker);
        if(!selectedVal.includes(value)) selectedVal.push(value);
    })

    $pickers.forEach(function($picker){
        let value   = $picker.$.value;
        let options = $picker.options;
        let items   = [];
        options.items.forEach(function(item)
        {
            item = $.extend({}, item, {disabled: false});
            if(value !== item.value && selectedVal.includes(item.value)) item = $.extend({}, item, {disabled: true});
            items.push(item);
        })
        options.items = items;
        delete options.afterRender;

        $picker.render(options);
    });
}

window.removeCondition = function(dom)
{
    if($(dom).hasClass('disabled')) return;
    $(dom).closest('.multicolumn-link-item').remove();

    disabledSelectedColumn();
    setLineIndex();
}

/**
 * 取消题目关联。
 * Disassociate the topic.
 *
 * @access public
 * @return void
 */
window.cancelLink = function()
{
    const url = $('#cancel-button').data('url');
    const confirm = $('#cancel-button').data('confirm');
    zui.Modal.confirm({message: confirm, icon: 'icon-exclamation-sign', iconClass: 'warning-pale rounded-full icon-2x'}).then((result) => {
        if(result) $.ajaxSubmit({url: url})
    })
}

/**
 * 选择问题节点。
 * Select a step.
 *
 * @access public
 * @return void
 */
window.selectQuestion = function()
{
    const stepID = $('[name="step"]').zui('picker').$.value;
    const {marketID, wizardID} = $('[name="step"]').closest('.form-group').data();
    loadModal($.createLink('thinkstep', 'link', `marketID=${marketID}&stepID=${stepID}&wizardID=${wizardID}&from=reload`));
}
