/**
 * 展示区块关联部分。
 * Display the blocks.
 *
 * @access public
 * @return void
 */
window.showBlocks = function()
{
    loadPage($.createLink('thinkwizard', 'result', `marketID=${marketID}&wizardID=${wizardID}&stepID=${stepID}&background=${$('[name="background"]').val()}`));
}

/**
 * 更改区块的选中值。
 * Change the selected value of the block.
 *
 * @param  object target
 * @param  int    order
 * @access public
 * @return void
 */
function changeBlock(target, order)
{
    const blockID = $(target).find('.pick-value').val();
    $(`.think-result-content .block-${order} .item-step-title`).text(steps[blockID] || unAssociatedLang);
}

/**
 * 改变区块选择器的下拉选项。
 * conver blocks picker item.
 *
 * @param  int     blockKey
 * @param  object  item
 * @access public
 * @return object
 */
window.convertBlockItem = function(blockKey, item)
{
    const blockID        = $('.picker-' + blockKey).zui('picker').$.value;
    const $blocks        = $('.blocks').find('.pick-value');
    const selectedBlocks = [];

    $blocks.each(function()
    {
        if($(this).val() != '' && $(this).val() != blockID) selectedBlocks.push($(this).val());
    });
    if(selectedBlocks.includes(item.value)) item.disabled = true;
}

/**
 * 提交分析步骤表单。
 * Submit think step form.
 *
 * @param  string   url
 * @param  function onSuccess
 * @access public
 * @return void
 */
function submitResultForm(url, onSuccess)
{
    const formData = new FormData($('.think-result form')[0]);
    $.ajaxSubmit({
        url: url,
        data: formData,
        load: false,
        onSuccess: onSuccess,
        onFail: (error) => {
            if(error?.message) showValidateMessage(error.message);
        }
    });
}

/**
 * 切换视图页面。
 * Switch view page.
 *
 * @access public
 * @return void
 */
window.switchViewOfResult = function()
{
    submitResultForm($.createLink('thinkwizard', 'result', `marketID=${marketID}&wizardID=${wizardID}&stepID=${stepID}`), () => {
        loadPage($.createLink('thinkstep', 'view', `marketID=${marketID}&wizardID=${wizardID}&stepID=${stepID}`));
    });
}

/**
 * 发布分析模型。
 * Publish think Model.
 *
 * @access public
 * @return void
 */
window.publishForResult = function()
{
    submitResultForm($.createLink('thinkwizard', 'result', `marketID=${marketID}&wizardID=${wizardID}&stepID=${stepID}`), () => {
        $.ajaxSubmit({url: $.createLink('thinkwizard', 'publish', `marketID=${marketID}&wizardID=${wizardID}&isJump=1&reload=0`)});
    });
}
