<?php
/**
 * 创建一个分析步骤。
 * Create a think step.
 *
 * @param  object $step
 * @param  object $wizard
 * @param  array  $steps
 * @access public
 * @return void
 */
public function initSingleStep($step, $wizard, $steps)
{
    $step->wizard      = $wizard->id;
    $step->createdBy   = $this->app->user->account;
    $step->createdDate = helper::now();

    $tableName = $this->config->db->prefix . 'thinkstep';
    if(isset($step->quoteTitle))
    {
        $qotedStep = $this->dao->select('*')->from("`$tableName`")
            ->where('title')->eq($step->quoteTitle)
            ->andWhere('wizard')->eq($wizard->id)
            ->fetch();
        $options = json_decode($step->options, true);
        $options['quoteTitle'] = $qotedStep->id;
        $step->options = json_encode($options, JSON_UNESCAPED_UNICODE);
        unset($step->quoteTitle);
    }
    $this->dao->insert("`$tableName`")->data($step, 'id')->exec();

    $step->id     = $this->dao->lastInsertID();
    $step->path   = $step->parent ? $steps[$step->parent]->path . $step->id . ',' : ',' . $step->id . ',';
    $step->parent = $step->parent ? $steps[$step->parent]->id : 0;
    $this->dao->update("`$tableName`")->set('path')->eq($step->path)->set('parent')->eq($step->parent)->where('id')->eq($step->id)->exec();
}

/**
 * 创建所有初始分析步骤。
 * Create all initial think steps.
 *
 * @param  object      $wizard
 * @param  string|null $version
 * @access public
 * @return void
 */
public function initThinkSteps($wizard, $version = null)
{
    if(!empty($version))
    {
        $initSteps = $this->config->thinkstep->initSteps[$version][$wizard->model];
        foreach($initSteps as $step) $this->initSingleStep($step, $wizard, $initSteps);
    }
    else
    {
        $allInitSteps = $this->config->thinkstep->initSteps;
        foreach($allInitSteps as $initSteps)
        {
            foreach($initSteps as $modelCode => $steps)
            {
                if($modelCode == $wizard->model)
                {
                    foreach($steps as $step) $this->initSingleStep($step, $wizard, $steps);
                }
            }
        }
    }
}

/**
 * 创建所有的初始分析模型。
 * Create all initial think wizards.
 *
 * @param  string|null $version
 * @access public
 * @return void
 */
public function initThinkWizards($version = null)
{
    $this->app->setVision();
    $this->app->loadConfig('thinkstep');
    if(!empty($version))
    {
        $initWizards = $this->config->thinkstep->initWizards[$version];
        foreach($initWizards as $wizard) $this->initSingleWizard($wizard, $version);
    }
    else
    {
        $initWizards = $this->config->thinkstep->initWizards;
        foreach($initWizards as $wizards)
        {
            foreach($wizards as $wizard) $this->initSingleWizard($wizard);
        }
    }
}

/**
 * 检查模型唯一性。
 * Check wizard unique.
 *
 * @param  object $wizard
 * @access public
 * @return bool
 */
public function checkWizardUnique($wizard)
{
    $tableName = $this->config->db->prefix . 'thinkwizard';
    $name      = is_array($wizard) ? $wizard['name'] : $wizard->name;
    $count     = $this->dao->select('COUNT(1) AS count')->from("`$tableName`")
        ->where('name')->eq($name)
        ->fetch('count');
    return $count == 0;
}

/**
 * 创建一个初始分析模型。
 * Create a think wizard.
 *
 * @param  object|array $wizard
 * @param  string|null  $version
 * @access public
 * @return void
 */
public function initSingleWizard($wizard, $version = null)
{
    $tableName = $this->config->db->prefix . 'thinkwizard';
    if($this->checkWizardUnique($wizard)) $this->dao->insert("`$tableName`")->data($wizard)->autoCheck()->exec();

    $wizardID = $this->dao->lastInsertID();
    $wizard   = $this->loadModel('thinkwizard')->getByID($wizardID);
    if($wizard)
    {
        $this->initThinkSteps($wizard, $version);
        if(!in_array($wizard->model, $this->config->thinkwizard->venn))
        {
            $steps      = $this->loadModel('thinkstep')->getPairs($wizard->id, 0, 1, '`order`');
            $stepIdList = array_keys($steps);
            $blocks     = json_encode($stepIdList, JSON_UNESCAPED_UNICODE);
            $this->dao->update("`$tableName`")->set('blocks')->eq($blocks)->where('id')->eq($wizard->id)->exec();
        }
    }
}
